#!/bin/bash

LOG_DIR="$HOME/log/$(date +%m-%d-%Y_%H-%M-%S)"
packagename="com.qq.weixin.deepin"
LOG_FILE=""
appname="微信"
ROOT_DIR=$(dirname $(realpath $0))

init_env()
{
    if [ -z "$BOX86_LOG" ];then
        export BOX86_LOG=1
    fi
    
    if [ -z "$BOX86_MODULE" ];then
        export BOX86_MODULE=1
    fi

    mkdir -p "$LOG_DIR/box86"
    export BOX86_LOG_DIR="$LOG_DIR/box86"

    if [ -n "$PACKAGE_NAME" ];then
        packagename="$PACKAGE_NAME"
    fi

    if [ -n "$APPNAME" ];then
        appname="$APPNAME"
    fi
}

init_log()
{
    date_str=$(date +%m-%d-%Y_%H-%M-%S)
    if [ ! -d "$LOG_DIR" ];then
        mkdir -p "$LOG_DIR"
    fi

    LOG_FILE="${LOG_DIR}/${packagename}_${date_str}.log"
}

print_system_info()
{
    lspci -vv &> "$LOG_DIR/pciinfo"
    lscpu &> "$LOG_DIR/cpuinfo"
    ip a &> "$LOG_DIR/ipinfo"
    cp /etc/deepin-version "$LOG_DIR/sysinfo"
    uname -a >> "$LOG_DIR/sysinfo"
    dpkg -l &> "$LOG_DIR/packageinfo"
    kill $(pidof box86)
    kill $(pidof box86.real)
}

enable_default_font_whitelist()
{
    if [ -n "$SAFE_FONTS" ];then
        bottlename=$(cat "/opt/apps/${packagename}/files/run.sh" | grep -E "^BOTTLENAME=" | awk -F = '{print $NF}' | awk -F \" '{print $2}')
        runner=$(cat "/opt/apps/${packagename}/files/run.sh" | grep -E "^export APPRUN_CMD=" | awk -F = '{print $NF}' | awk -F \" '{print $2}')
        WINEPREFIX="$HOME/.deepinwine/$bottlename" WINE="$runner" "$ROOT_DIR/set_font_whitelist.sh"
    fi
}

run_app()
{
    EXE_NAME=$(cat /opt/apps/${packagename}/files/run.sh | grep EXE=)
    if [ -z "$EXE_NAME" ];then
        EXE_NAME=$(cat /opt/apps/${packagename}/files/run.sh | grep EXEC_PATH=)
    fi
    EXE_NAME=$(echo $EXE_NAME | awk -F '"' '{print $2}' | awk -F / '{print $NF}')
    echo "启动$appname $packagename"

    if [ ! -f "/opt/apps/${packagename}/files/run.sh" ];then
        echo "$packagename 未安装"
        exit
    fi

    reset_init_bottle_flag

    echo "WINEDEBUG=$WINEDEBUG" > "$LOG_FILE"
    enable_default_font_whitelist &>> "$LOG_FILE"
    /opt/apps/${packagename}/files/run.sh &>> "$LOG_FILE"

    echo "等待$appname $EXE_NAME 启动"

    start_time=$(date +%s)
    EXE_PID=""
    while [ -z "$EXE_PID" ];do
        sleep 1
        EXE_PID=$(ps -ef | grep $EXE_NAME | grep -v grep)
        if [ `expr $(date +%s) - $start_time` -gt 10 ];then
            return
        fi
    done
}

save_meminfo()
{
    MEM_INFO_DIR="$LOG_DIR/meminfo"
    if [ ! -d "$MEM_INFO_DIR" ];then
        mkdir -p "$MEM_INFO_DIR"
    fi

    let index=0
    for n in {20..1};do
        if [ -f "${MEM_INFO_DIR}/_pre_${n}s" ];then
            index=$n
            break
        fi
    done

    for (( ;index>0;index--));do
        src_log="${MEM_INFO_DIR}/_pre_${index}s"
        let dest=index+1
        dest_log="${MEM_INFO_DIR}/_pre_${dest}s"
        mv "$src_log" "$dest_log"
    done

    top -n 1 -o %MEM -b &> "${MEM_INFO_DIR}/_pre_1s"
}

save_procinfo()
{
    PROC_INFO_DIR="$LOG_DIR/procinfo"
    if [ ! -d "$PROC_INFO_DIR" ];then
        mkdir -p "$PROC_INFO_DIR"
    fi

    let index=0
    for n in {20..1};do
        if [ -f "${PROC_INFO_DIR}/_pre_${n}s" ];then
            index=$n
            break
        fi
    done

    for (( ;index>0;index--));do
        src_log="${PROC_INFO_DIR}/_pre_${index}s"
        let dest=index+1
        dest_log="${PROC_INFO_DIR}/_pre_${dest}s"
        mv "$src_log" "$dest_log"
    done

    ps -ef > "${PROC_INFO_DIR}/_pre_1s"

    proc_info_files=(environ limits cmdline maps)
    ps -ef | grep -i -E "$USER.*\.exe"| grep -v grep | awk '{print $2}' | while read winepid;do
        if [ -d "${PROC_INFO_DIR}/wine/$winepid" ];then
            rm -rf "${PROC_INFO_DIR}/wine/$winepid"
        fi
        mkdir -p "${PROC_INFO_DIR}/wine/$winepid"
        for filename in "${proc_info_files[@]}" ;do
            cp "/proc/$winepid/$filename" "${PROC_INFO_DIR}/wine/$winepid"
        done
    done
}

wait_quit()
{
    echo "等待$appname $EXE_NAME 退出"
    start_time=$(date +%s)
    EXE_PID="fffff"
    rm /tmp/deepin-wine-diag
    while [ -n "$EXE_PID" ];do
        beQuit=$(cat /tmp/deepin-wine-diag)
        if [ "$beQuit" == "quit" ];then
            mv /tmp/deepin-wine-diag "$LOG_DIR"
            return
        fi
        save_procinfo
        save_meminfo
        sleep 1
        ps -ef > "${PROC_INFO_DIR}/quit"
        top -n 1 -o %MEM -b &> "${MEM_INFO_DIR}/quit"
        EXE_PID=$(cat "${PROC_INFO_DIR}/quit" | grep $EXE_NAME | grep -v grep)
    done
    echo "$appname $EXE_NAME 退出"
}

get_bottle_name()
{
    bottle_name=$(cat "/opt/apps/${packagename}/files/run.sh" | grep BOTTLENAME=)

    if [ -n "$bottle_name" ];then
        bottle_name=$(echo $bottle_name| awk -F = '{print $NF}' | awk -F \" '{print $2}')
    else
        bottle_name=$(cat "/opt/apps/${packagename}/files/run.sh" | grep BOTTLE=)
        bottle_name=$(echo $bottle_name| awk -F = '{print $NF}' | awk -F \" '{print $2}')
        bottle_name=$(echo $bottle_name | awk -F / '{print $NF}')
    fi
}

reset_init_bottle_flag()
{
    get_bottle_name

    rm "$HOME/.deepinwine/$bottle_name/.init_"*
}

cp_tencent_temp()
{
    get_bottle_name

    cp -a "$HOME/.deepinwine/$bottle_name/drive_c/users/$USER/Temp/Tencent/" "$LOG_DIR"
}

compress_log()
{
    if [ -f "$LOG_DIR/deepin-wine-diag" ];then
        beQuit=$(cat "$LOG_DIR/deepin-wine-diag")
        if [ "$beQuit" == "quit" ];then
            return
        fi
    fi

    mkdir "${LOG_DIR}/coredump"
    find /var/lib/systemd/coredump  -maxdepth 1 -mtime -2 -exec cp --preserve=timestamp '{}' "${LOG_DIR}/coredump" \;

    compress_dir=$(realpath "$HOME/Desktop")
    compress_name="${compress_dir}/${packagename}_$(date +%m-%d-%Y_%H-%M-%S).7z"
    echo "压缩日志中... $compress_name"
    7z a "$compress_name" "$LOG_DIR"
    dde-file-manager --show-item "$compress_name" &
}

init_env
init_log
print_system_info
run_app
wait_quit
cp_tencent_temp
compress_log
rm -rf "$LOG_DIR"
